﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Test
{
    public class Example : UserControl
    {
        //protected static readonly WindowMain MainWindow = System.Windows.Application.Current.MainWindow as WindowMain;
        internal static Dictionary<Type, string> Descriptions =
                new Dictionary<Type, string> 
			{
       

				{ typeof(Window2), "Polar chart used as speedometer gauge." }, 

         


          

			};

        internal static Example Create(Type typeOfExample)
        {
            bool b = typeof(Example).IsAssignableFrom(typeOfExample);
            if (b == true)
            {
                return Activator.CreateInstance(typeOfExample) as Example;
            }

            return null;
        }

        protected Example()
        {
        }

        ~Example()
        {
            System.Diagnostics.Debug.WriteLine("Example was not disposed of. Do not forget to call example's Dispose method to release any unmanaged resources.");
        }

        internal virtual string GetDescription(out string className)
        {
            // If we end up here, the example we are getting a description 
            // for does not have an overridden GetDescription method. Let's 
            // use ExampleDescriptions class' Descriptions dictionary to get 
            // a description for the example.

            string description;

            Type type = GetType();
            try
            {
                description = Example.Descriptions[type];
            }
            catch
            {
                description = "No description available.";
            }

            className = type.Name;

            return description;
        }


        internal delegate void ChartsCreatedEventHandler();
        internal event ChartsCreatedEventHandler ChartsCreated;
        public void RaiseChartsCreatedEvent()
        {
            if (ChartsCreated != null)
            {
                ChartsCreated();
            }
        }
        public class RealtimeExample : Example
        {

            public delegate void DataGeneratedEventHandler(long datapointCount);
            public delegate void StartedEventHandler();
            public delegate void StoppedEventHandler();

            public event RealtimeExample.StartedEventHandler Started;
            public event RealtimeExample.StoppedEventHandler Stopped;
            public event RealtimeExample.DataGeneratedEventHandler DataGenerated;


            private long m_lTotalDatapointCount;

            protected RealtimeExample()
            {
                m_lTotalDatapointCount = 0;
            }

            internal virtual bool IsRunning
            {
                get
                {
                    throw new Exception("RealtimeExample.IsRunning property cannot be accessed directly. Override it in inherited class.");
                }
            }

            public virtual void Start()
            {
                throw new Exception("RealtimeExample.Start method cannot be called directly. Override it in inherited class.");
            }

            public virtual void Stop()
            {
                throw new Exception("RealtimeExample.Stop method cannot be called directly. Override it in inherited class.");
            }


            protected void RaiseStartedEvent()
            {
                m_lTotalDatapointCount = 0;

                if (Started != null)
                {
                    Started();
                }
            }

            protected void RaiseStoppedEvent()
            {
                if (Stopped != null)
                {
                    Stopped();
                }
            }

            protected void RaiseDataGeneratedEvent(long datapointCount)
            {
                m_lTotalDatapointCount += datapointCount;

                if (DataGenerated != null)
                {
                    DataGenerated(m_lTotalDatapointCount);
                }
            }

            protected void RaiseDataGeneratedEvent(long datapointCount, bool overwriteCounter)
            {
                if (overwriteCounter == true)
                {
                    m_lTotalDatapointCount = datapointCount;
                }
                else
                {
                    m_lTotalDatapointCount += datapointCount;
                }

                if (DataGenerated != null)
                {
                    DataGenerated(m_lTotalDatapointCount);
                }
            }

        }
    }
}
